@@ -15,6 +15,8 @@
 #include "gfx_type.h"
 #include "strings_type.h"
 #include "landscape_type.h"
+#include "core/bitmath_func.hpp"
+#include "core/span_type.hpp"
 #include <vector>
 
 /** Globally unique label of a cargo type. */
@@ -59,7 +61,7 @@ struct CargoSpec {
 	uint8 rating_colour;
 	uint8 weight;                    ///< Weight of a single unit of this cargo type in 1/16 ton (62.5 kg).
 	uint16 multiplier;               ///< Capacity multiplier for vehicles. (8 fractional bits)
-	uint16 initial_payment;
+	int32 initial_payment;           ///< Initial payment rate before inflation is applied.
 	uint8 transit_days[2];
 
 	bool is_freight;                 ///< Cargo type is considered to be freight (affects train freight multiplier).
@@ -122,6 +124,49 @@ struct CargoSpec {
 
 	SpriteID GetCargoIcon() const;
 
+	/**
+	 * Iterator to iterate all valid CargoSpec
+	 */
+	struct Iterator {
+		typedef CargoSpec value_type;
+		typedef CargoSpec *pointer;
+		typedef CargoSpec &reference;
+		typedef size_t difference_type;
+		typedef std::forward_iterator_tag iterator_category;
+
+		explicit Iterator(size_t index) : index(index)
+		{
+			this->ValidateIndex();
+		};
+
+		bool operator==(const Iterator &other) const { return this->index == other.index; }
+		bool operator!=(const Iterator &other) const { return !(*this == other); }
+		CargoSpec * operator*() const { return CargoSpec::Get(this->index); }
+		Iterator & operator++() { this->index++; this->ValidateIndex(); return *this; }
+
+	private:
+		size_t index;
+		void ValidateIndex() { while (this->index < CargoSpec::GetArraySize() && !(CargoSpec::Get(this->index)->IsValid())) this->index++; }
+	};
+
+	/*
+	 * Iterable ensemble of all valid CargoSpec
+	 */
+	struct IterateWrapper {
+		size_t from;
+		IterateWrapper(size_t from = 0) : from(from) {}
+		Iterator begin() { return Iterator(this->from); }
+		Iterator end() { return Iterator(CargoSpec::GetArraySize()); }
+		bool empty() { return this->begin() == this->end(); }
+	};
+
+	/**
+	 * Returns an iterable ensemble of all valid CargoSpec
+	 * @param from index of the first CargoSpec to consider
+	 * @return an iterable ensemble of all valid CargoSpec
+	 */
+	static IterateWrapper Iterate(size_t from = 0) { return IterateWrapper(from); }
+
 private:
 	static CargoSpec array[NUM_CARGO]; ///< Array holding all CargoSpecs
 
@@ -134,10 +179,11 @@ extern CargoTypes _standard_cargo_mask;
 void SetupCargoForClimate(LandscapeID l);
 CargoID GetCargoIDByLabel(CargoLabel cl);
 CargoID GetCargoIDByBitnum(uint8 bitnum);
+CargoID GetDefaultCargoID(LandscapeID l, CargoType ct);
 
 void InitializeSortedCargoSpecs();
 extern std::vector<const CargoSpec *> _sorted_cargo_specs;
-extern uint8 _sorted_standard_cargo_specs_size;
+extern span<const CargoSpec *> _sorted_standard_cargo_specs;
 
 /**
  * Does cargo \a c have cargo class \a cc?
@@ -150,24 +196,6 @@ static inline bool IsCargoInClass(CargoID c, CargoClass cc)
 	return (CargoSpec::Get(c)->classes & cc) != 0;
 }
 
-#define FOR_ALL_CARGOSPECS_FROM(var, start) for (size_t cargospec_index = start; var = nullptr, cargospec_index < CargoSpec::GetArraySize(); cargospec_index++) \
-		if ((var = CargoSpec::Get(cargospec_index))->IsValid())
-#define FOR_ALL_CARGOSPECS(var) FOR_ALL_CARGOSPECS_FROM(var, 0)
-
-#define FOR_EACH_SET_CARGO_ID(var, cargo_bits) FOR_EACH_SET_BIT_EX(CargoID, var, CargoTypes, cargo_bits)
-
-/**
- * Loop header for iterating over cargoes, sorted by name. This includes phony cargoes like regearing cargoes.
- * @param var Reference getting the cargospec.
- * @see CargoSpec
- */
-#define FOR_ALL_SORTED_CARGOSPECS(var) for (uint8 index = 0; index < _sorted_cargo_specs.size() && (var = _sorted_cargo_specs[index], true) ; index++)
-
-/**
- * Loop header for iterating over 'real' cargoes, sorted by name. Phony cargoes like regearing cargoes are skipped.
- * @param var Reference getting the cargospec.
- * @see CargoSpec
- */
-#define FOR_ALL_SORTED_STANDARD_CARGOSPECS(var) for (uint8 index = 0; index < _sorted_standard_cargo_specs_size && (var = _sorted_cargo_specs[index], true); index++)
+using SetCargoBitIterator = SetBitIterator<CargoID, CargoTypes>;
 
 #endif /* CARGOTYPE_H */
