@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -34,17 +32,15 @@ static const CommandCost CMD_ERROR = CommandCost(INVALID_STRING_ID);
  */
 #define return_cmd_error(errcode) return CommandCost(errcode);
 
-CommandCost DoCommand(TileIndex tile, uint32 p1, uint32 p2, DoCommandFlag flags, uint32 cmd, const char *text = NULL);
+CommandCost DoCommand(TileIndex tile, uint32 p1, uint32 p2, DoCommandFlag flags, uint32 cmd, const std::string &text = {});
 CommandCost DoCommand(const CommandContainer *container, DoCommandFlag flags);
 
-bool DoCommandP(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback = NULL, const char *text = NULL, bool my_cmd = true);
+bool DoCommandP(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback = nullptr, const std::string &text = {}, bool my_cmd = true);
 bool DoCommandP(const CommandContainer *container, bool my_cmd = true);
 
-CommandCost DoCommandPInternal(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback, const char *text, bool my_cmd, bool estimate_only);
+CommandCost DoCommandPInternal(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback, const std::string &text, bool my_cmd, bool estimate_only);
 
-#ifdef ENABLE_NETWORK
-void NetworkSendCommand(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback, const char *text, CompanyID company);
-#endif /* ENABLE_NETWORK */
+void NetworkSendCommand(TileIndex tile, uint32 p1, uint32 p2, uint32 cmd, CommandCallback *callback, const std::string &text, CompanyID company);
 
 extern Money _additional_cash_required;
 
@@ -81,7 +77,7 @@ CommandCallback CcBuildBridge;
 
 /* dock_gui.cpp */
 CommandCallback CcBuildDocks;
-CommandCallback CcPlaySound_SPLAT_WATER;
+CommandCallback CcPlaySound_CONSTRUCTION_WATER;
 
 /* depot_gui.cpp */
 CommandCallback CcCloneVehicle;
@@ -100,16 +96,15 @@ CommandCallback CcBuildIndustry;
 CommandCallback CcPlaySound_EXPLOSION;
 CommandCallback CcPlaceSign;
 CommandCallback CcTerraform;
-CommandCallback CcGiveMoney;
 
 /* rail_gui.cpp */
-CommandCallback CcPlaySound_SPLAT_RAIL;
+CommandCallback CcPlaySound_CONSTRUCTION_RAIL;
 CommandCallback CcRailDepot;
 CommandCallback CcStation;
 CommandCallback CcBuildRailTunnel;
 
 /* road_gui.cpp */
-CommandCallback CcPlaySound_SPLAT_OTHER;
+CommandCallback CcPlaySound_CONSTRUCTION_OTHER;
 CommandCallback CcBuildRoadTunnel;
 CommandCallback CcRoadDepot;
 CommandCallback CcRoadStop;
