@@ -13,6 +13,7 @@
 #ifndef NEWGRF_TOWNNAME_H
 #define NEWGRF_TOWNNAME_H
 
+#include <vector>
 #include "strings_type.h"
 
 struct NamePart {
@@ -45,9 +46,11 @@ GRFTownName *AddGRFTownName(uint32 grfid);
 GRFTownName *GetGRFTownName(uint32 grfid);
 void DelGRFTownName(uint32 grfid);
 void CleanUpGRFTownNames();
-StringID *GetGRFTownNameList();
 char *GRFTownNameGenerate(char *buf, uint32 grfid, uint16 gen, uint32 seed, const char *last);
 uint32 GetGRFTownNameId(int gen);
 uint16 GetGRFTownNameType(int gen);
+StringID GetGRFTownNameName(uint gen);
+
+const std::vector<StringID>& GetGRFTownNameList();
 
 #endif /* NEWGRF_TOWNNAME_H */
