@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -40,4 +38,17 @@ bool IsMonospaceFont(CFStringRef name);
 
 void MacOSSetThreadName(const char *name);
 
+
+/** Deleter that calls CFRelease rather than deleting the pointer. */
+template <typename T> struct CFDeleter {
+	void operator()(T *p)
+	{
+		if (p) ::CFRelease(p);
+	}
+};
+
+/** Specialisation of std::unique_ptr for CoreFoundation objects. */
+template <typename T>
+using CFAutoRelease = std::unique_ptr<typename std::remove_pointer<T>::type, CFDeleter<typename std::remove_pointer<T>::type>>;
+
 #endif /* MACOS_H */
