@@ -1,5 +1,3 @@
-/* $Id$ */
-
 /*
  * This file is part of OpenTTD.
  * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
@@ -76,10 +74,8 @@ enum Ratings {
 	RATING_BRIBE_DOWN_TO = -50        // XXX SHOULD BE SOMETHING LOWER?
 };
 
-/**
- * Town Layouts
- */
-enum TownLayout {
+/** Town Layouts. It needs to be 8bits, because we save and load it as such */
+enum TownLayout : byte {
 	TL_BEGIN = 0,
 	TL_ORIGINAL = 0,     ///< Original algorithm (min. 1 distance between roads)
 	TL_BETTER_ROADS,     ///< Extended original algorithm (min. 2 distance between roads)
@@ -91,19 +87,23 @@ enum TownLayout {
 	NUM_TLS,             ///< Number of town layouts
 };
 template <> struct EnumPropsT<TownLayout> : MakeEnumPropsT<TownLayout, byte, TL_BEGIN, NUM_TLS, NUM_TLS, 3> {};
-/** It needs to be 8bits, because we save and load it as such */
-typedef SimpleTinyEnumT<TownLayout, byte> TownLayoutByte; // typedefing-enumification of TownLayout
 
-/** Town founding setting values */
-enum TownFounding {
+/** Town founding setting values. It needs to be 8bits, because we save and load it as such */
+enum TownFounding : byte {
 	TF_BEGIN = 0,     ///< Used for iterations and limit testing
 	TF_FORBIDDEN = 0, ///< Forbidden
 	TF_ALLOWED,       ///< Allowed
 	TF_CUSTOM_LAYOUT, ///< Allowed, with custom town layout
 	TF_END,           ///< Used for iterations and limit testing
 };
-/** It needs to be 8bits, because we save and load it as such */
-typedef SimpleTinyEnumT<TownFounding, byte> TownFoundingByte;
+
+/** Town cargo generation modes */
+enum TownCargoGenMode : byte {
+	TCGM_BEGIN = 0,
+	TCGM_ORIGINAL = 0,  ///< Original algorithm (quadratic cargo by population)
+	TCGM_BITCOUNT,      ///< Bit-counted algorithm (normal distribution from individual house population)
+	TCGM_END,
+};
 
 static const uint MAX_LENGTH_TOWN_NAME_CHARS = 32; ///< The maximum length of a town name in characters including '\0'
 
